#include <stdio.h>
#include <tk.h>

static Tcl_CmdProc LoadPatternCmd;

/* this function is dynamically loaded by the "load" command */
Loadpattern_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "load_pattern", LoadPatternCmd, NULL, NULL);
  return(TCL_OK);
}

/* this C function implements the "load_pattern" command */
static int LoadPatternCmd(cd, interp, argc, argv)
     ClientData cd;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  char *photo_name;
  Tk_PhotoHandle p_hand;
  Tk_PhotoImageBlock block;
  int width, height, x, y;
  int val;
  unsigned char *image;

  if(argc < 3){
    Tcl_AppendResult(interp, "wrong # args", (char *)NULL);
    return(TCL_ERROR);
  }

  photo_name = argv[1];  /* name of photo in which to display */
  val = atoi(argv[2]);   /* test pattern number */

  p_hand = Tk_FindPhoto(photo_name);
  if(p_hand ==  NULL){
    Tcl_AppendResult(interp, "can't find photo named \"",
		     photo_name, "\"", (char *)NULL);
    return(TCL_ERROR);
  }
  
  Tk_PhotoGetSize(p_hand, &width, &height);
  
  image = (unsigned char *)malloc(width * height * sizeof(unsigned char));
  if(image == NULL){
    Tcl_AppendResult(interp, 
		     "can't allocate image memory",
		     (char *)NULL);
    return(TCL_ERROR);
  }
   
  /* generate a simple pattern.  Neither interesting nor efficient. */
  for(y = 0; y < height; y++)
    for(x = 0; x < width; x++)
      image[y*width + x] = (x % (256/val))*val;
      
  /* describe image memory layout to photo */

  /* first, a pointer to the data */
  block.pixelPtr = image; 

  /* the dimensions of the data */
  block.width = width;
  block.height = height;

  /* number of bytes between adjacent scanlines */
  block.pitch = width;

  /* number of bytes of one pixel */
  block.pixelSize = 1;

  /* byte offsets to each of three color bands (r, g, b).  In this case,
     the image is grayscale, so the offsets of r = g = b.  */
  block.offset[0] = block.offset[1] = block.offset[2] = 0; 

  /* send the bytes */
  Tk_PhotoPutBlock(p_hand, &block, 0, 0, width, height);

  /* we don't need our copy any more */
  free((void *)image);
  return(TCL_OK);
}
